#include <iostream.h>
#include <fstream.h>

inline int min ( int a, int b ) {
	return ( a < b ) ? ( a ) : ( b );
}

struct CLink {
	int b, Q;
	double P;
};

struct CNode {
	int l;
	bool visited;
	CLink * links;
	CNode ( void ) {
		visited = false;
	}
	~CNode ( void ) {
		delete [] links;
	}
};

class CNetwork {
private:
	int n, s, e;
	CNode * nodes;
	int * solution;
	int count;
	int * best;
	int bestCount;
	double bestPQ;
public:
	void Start ( void );
	~CNetwork ( void );
private:
	void ReadData ( void );
	void Go ( int nNode, int nQ, double dP ); 
	void WriteData ( void );
};

void CNetwork::Start ( void ) {
	ReadData ();
	Go ( s, 65535, 1 );
	WriteData ();
}

void CNetwork::ReadData ( void ) {
	ifstream input ( "network.inp", ios::in );
	int i, m, a, b, Q;
	double P;

	input >> n >> m;
	nodes = new CNode [n];
	for ( i = 0; i < n; ++i ) {
		nodes[i].l = 0;
		nodes[i].links = new CLink [n];
	}
	for ( i = 0; i < m; ++i ) {
		input >> a >> b >> Q >> P;
		--a; --b;
		nodes[a].links[nodes[a].l].b = b;
		nodes[a].links[nodes[a].l].Q = Q;
		nodes[a].links[nodes[a].l].P = P;
		nodes[a].l++;
	}
	input >> s >> e;
	--s; --e;
	solution = new int [n];
	count = 0;
	bestCount = 0;
	best = new int [n];
	bestPQ = 0.0;
}

void CNetwork::Go ( int nNode, int nQ, double dP ) {
	if ( nodes[nNode].visited == false && bestPQ <nQ * dP ) {
		nodes[nNode].visited = true;
		solution[count++] = nNode;
		if	( nNode == e ) {
			bestCount = count;
			for ( int i = 0; i < count; ++i )
				best[i] = solution[i];
			bestPQ = nQ * dP;
		}
		else {
			int i;

			for ( i = 0; i < nodes[nNode].l; ++i ) {
				Go ( nodes[nNode].links[i].b, min ( nQ, nodes[nNode].links[i].Q ), dP * nodes[nNode].links[i].P );
			}
		}
		--count;
		nodes[nNode].visited = false;
	}
}

void CNetwork::WriteData ( void ) {
	ofstream output ( "network.out", ios::out );
	int i;

	output.setf ( ios::fixed );
	output.precision ( 6 );

	if ( bestPQ == 0 ) {
		output << 0 << ' ' << 0 << endl;
	}
	else {
		output << bestPQ << ' ' << bestCount << endl;
		for ( i = 0; i < bestCount; ++i )
			output << best[i] + 1 << ' ';
	}
}

CNetwork::~CNetwork ( void ) {
	delete [] solution;
	delete [] nodes;
	delete [] best;
}

void main ( void ) {
	CNetwork obj;
	obj.Start ();
}
