/*
Kompilator: Visual C++ 6.0
Avtor: Petko Minkov
Vyzrast: 18
Zanimanie: Uchenik
Grad: Plovdiv
*/
#include <search.h>
#include <stdio.h>
#include <fstream.h>
 
#define max(a,b) (a > b ? a : b)

#define Gv(s, i) (g[s].ar[i].v)
#define Gq(s, i) (g[s].ar[i].q)
#define Gp(s, i) (g[s].ar[i].p)

#define INF 200000
#define MAXW 70000
#define MAXN 1005

typedef unsigned int uint;

struct adjvert
{
	int   v;
	uint  q;
	double p;
};

struct vertex
{
	adjvert ar[MAXN];
	int count;
};

char   ew[MAXW];
int    ewlist[MAXW];
int    ewlistcount;
int    pred[MAXN];
int    qoptimal;
vertex g[MAXN];
double dist[MAXN];
double dijkres[MAXW];
int    vis[MAXN];
int    n,m,s,t,savepath;
double ans;

/* Reads input file and stores graph in memory */
void getdata()
{
	int    i,u,v,q;
	double p;

	ifstream fin("network.inp");
	fin >> n >> m;
	for(i=1;i<=m;i++)
	{
		fin >> u >> v >> q >> p;
		g[u].count++;
		Gp(u, g[u].count) = p;
		Gq(u, g[u].count) = q;
		Gv(u, g[u].count) = v;
		ew[q] = 1;
	}
	fin >> s >> t;
	fin.close();
}

/* Compares two vertices (used in qsort) */
int cmpvert(const void *ac, const void *bc)
{
	adjvert a,b;

	a = *(adjvert*)ac;
	b = *(adjvert*)bc;

	if(a.q > b.q) return -1;
	if(a.q < b.q) return 1;
	return 0;
}

/* Sort the edges of the graph using qsort for every vertex */
void sortgraph()
{
	int i;

	for(i=1;i<=n;i++)
	{
		g[i].ar[0].q = 65535;
		qsort((void*) g[i].ar, g[i].count+1, sizeof(adjvert), cmpvert);
	}
}

int zero(double a) {
	return (a < 0 ? -a : a) < 0.000001;
}

double min(unsigned int a,unsigned int b) {
	return a < b ? a : b;
}

/* Uses Dijkstra algorithm to calculate optimal path */
double calculatepath(uint qlim)
{
	int    i,j,w;
	double maxdist;

	if(dijkres[qlim] != -1) return dijkres[qlim];

	for(i=1;i<=n;i++)
	{ 
		vis[i] = 0; 
		dist[i] = 0; 
		if(savepath) pred[i] = 0; 
	}
   
	for(i=1;i<=g[s].count;i++)
		if(Gq(s,i) >= qlim)
		{
			dist[Gv(s,i)] = Gp(s,i);
			if(savepath) pred[Gv(s,i)] = s;
		}
	
	dist[s] = 10;
	vis[s] = 1;
	for(i=1;i<=n-1;i++)
	{
		maxdist = 0;
		
		for(j=1;j<=n;j++)
			if(dist[j] > maxdist && !vis[j])
			{
				maxdist = dist[j];
				w = j;
			}
		
		if(zero(maxdist)) break;

		vis[w] = 1;

		for(j=1;j<=g[w].count;j++) if(Gq(w,j) >= qlim)
		{
			if(dist[Gv(w,j)] < Gp(w,j) * dist[w])
			{
				dist[Gv(w,j)] = Gp(w,j) * dist[w];
				if(savepath) pred[Gv(w,j)] = w; 
			}
		} else break;
	}

	dijkres[qlim] = dist[t];
	return dist[t];
}


/* This one cuts a lot of time */
void optimize(int left,int right)
{
	int    mid;
	double leftres, rightres;

	if(double(ewlist[left]) <= ans && double(ewlist[right]) <= ans) 
		return;

	leftres = calculatepath(ewlist[left]);
	rightres = calculatepath(ewlist[right]);

	if(ans < double(ewlist[left]) * leftres)
	{
		qoptimal = ewlist[left];
		ans = double(ewlist[left]) * leftres;
	}

	if(ans < double(ewlist[right]) * rightres)
	{
		qoptimal = ewlist[right];
		ans = double(ewlist[right]) * rightres;
	}

	if(zero(leftres - rightres) || left + 1 == right) 
		return;

	mid = (left+right) / 2;

	optimize(left, mid);
	optimize(mid, right);
}

/* Main task function */
void solveit()
{
	long     i,j;
	int	     stack[MAXN];
	int	     sind;	
	char     tmps[255];
	ofstream fout("network.out");

	for(i=MAXW-1;i>=1;i--) if(ew[i])
		ewlist[++ewlistcount] = i;

	for(i=0;i<MAXW;i++) dijkres[i] = -1;

	optimize(1, ewlistcount);

	if(zero(ans))
	{
		fout << "0 0\n";
		fout.close();
		return;
	}

	savepath = 1;
	dijkres[qoptimal] = -1; 
	calculatepath(qoptimal);

	sind = 0;
	while(t)
	{
		stack[++sind] = t;
		t = pred[t];
	}

	// Output stuff
	sprintf(tmps,"%.6lf",ans);

	fout << tmps << " " << sind << endl;
	
	for(i=sind;i>=1;i--)
		fout << stack[i] << " ";

	fout << endl;

	fout.close();
}

void main()
{
	getdata();
	sortgraph();
	solveit();
}