// Network.cpp : Defines the entry point for the application.
//

#include "stdafx.h"

#include <process.h>
#include <memory.h>
#include <stdio.h>

unsigned long NumRouters,NumDataLines,NumSourceDataLines,Source,Destination;
long double minE=0.0;

struct DataLine {
    unsigned short Source,Destination,Q;
    long double P,E;
    bool active;
} *DataLines,*SourceDataLines;

struct NeighborList {
    unsigned short Neighbor;
    unsigned short Q;
    unsigned long Distance;
    long double P;
    NeighborList *next;
};

struct NeighborData {
    bool neighbors;
    unsigned short Q;
    long double P;
} NeighborTab[1000][1000];

struct EfficiencyList {
    unsigned long Distance;
    unsigned long Changed;
    long double P;
    long double E;
    unsigned short Q;
    unsigned short Source;
    unsigned short SourceQ;
    EfficiencyList *next;
} *Table[1000];

bool IsSrc[1000],IsDest[1000];

void Print00()
{
    FILE *ouf;

    ouf=fopen("NETWORK.OUT","wt");

    fprintf(ouf,"0 0\n");

    fclose(ouf);

    exit(0);
}

void ReadData()
{
    unsigned long i;
    unsigned long s,d,q;
    long double p;
    FILE *inf; // Using files instead of streams makes the program really faster

    inf=fopen("NETWORK.INP","rt");

    fscanf(inf,"%lu %lu",&NumRouters,&NumDataLines);

    DataLines=new DataLine[NumDataLines];

    for(i=0;i<NumDataLines;i++) {
        fscanf(inf,"%lu %lu %lu %Lf",&s,&d,&q,&p);

        DataLines[i].Source=(unsigned short)--s;
        DataLines[i].Destination=(unsigned short)--d;
        DataLines[i].P=p;
        DataLines[i].Q=(unsigned short)q;
        DataLines[i].E=p*q;

        NeighborTab[s][d].neighbors=true;
        NeighborTab[s][d].P=p;
        NeighborTab[s][d].Q=(unsigned short)q;

        IsSrc[s]=true;
        IsDest[d]=true;
    }

    fscanf(inf,"%lu %lu",&Source,&Destination);

    Source--;
    Destination--;

    IsSrc[Destination]=true;
    IsDest[Source]=true;

    fclose(inf);
}

void FindMinE()
{
    NeighborList *first,*last,*p;
    NeighborData *nd;
    bool Found=false;
    unsigned long c,i,pn;

    if(Source==Destination) Print00();

    first=last=new NeighborList;

    first->Distance=0;
    first->Neighbor=(unsigned short)Source;
    first->P=1.0;
    first->Q=65535;
    first->next=0;

    while(first) {
        p=first;
        first=first->next;

        pn=p->Neighbor;

        for(c=0;(c<NumRouters)&&(!NeighborTab[pn][c].neighbors);c++);

        if((c<NumRouters)&&(!first)) {
            nd=&NeighborTab[pn][c];

            for(i=0;i<NumRouters;i++) NeighborTab[i][c].neighbors=false;

            first=last=new NeighborList;
            first->Distance=p->Distance+1;
            first->Neighbor=(unsigned short)c;
            first->Q=(nd->Q<p->Q?nd->Q:p->Q);
            first->P=nd->P*p->P;
            first->next=0;

            if(c==Destination) {
                delete p;

                minE=first->Q*first->P;

                break;
            }

            for(c++;(c<NumRouters)&&(!NeighborTab[pn][c].neighbors);c++);
        }

        while(c<NumRouters) {
            nd=&NeighborTab[pn][c];

            for(i=0;i<NumRouters;i++) NeighborTab[i][c].neighbors=false;

            last->next=new NeighborList;
            last=last->next;

            last->Distance=p->Distance+1;
            last->Neighbor=(unsigned short)c;
            last->Q=(nd->Q<p->Q?nd->Q:p->Q);
            last->P=nd->P*p->P;
            last->next=0;

            if(c==Destination) {
                delete p;

                minE=last->Q*last->P;

                break;
            }

            for(c++;(c<NumRouters)&&(!NeighborTab[pn][c].neighbors);c++);
        }

        if(minE!=0.0) break;

        delete p;
    }

    while(first) {
        p=first;
        first=first->next;

        delete p;
    }

    if(minE==0.0) Print00();
}

void Optimize()
{
    unsigned long i,cs,cd;

    for(i=0;i<NumDataLines;i++) if(DataLines[i].Source==Source) NumSourceDataLines++;

    SourceDataLines=new DataLine[NumSourceDataLines];

    for(i=cs=cd=0;i<NumDataLines;i++) {
        DataLines[i].active=true;

        if(DataLines[i].Source==Source) {
            if(DataLines[i].E>=minE) SourceDataLines[cs++]=DataLines[i];
            else NumSourceDataLines--;
        }
        else {
            if((DataLines[i].Destination==Source)||(DataLines[i].P*DataLines[i].Q<minE)) continue;

            if(!IsSrc[DataLines[i].Destination]) continue;
            if(!IsDest[DataLines[i].Source]) continue;

            DataLines[cd++]=DataLines[i];
        }
    }

    NumDataLines=cd;
}

void Solve()
{
    unsigned long i,j;
    unsigned long s,d,q;
    unsigned long cc=2; // Current change
    unsigned long pc=1; // Previous change
    long double p;
    bool Change=true;
    EfficiencyList *el,**pel,*el2,**pel2;

    for(i=0;i<NumSourceDataLines;i++) {
        pel=&Table[SourceDataLines[i].Destination];
        q=SourceDataLines[i].Q;

        while((*pel)&&((*pel)->next)&&((*pel)->next->Q<q)) pel=&((*pel)->next);
        if((*pel)&&((*pel)->Q<q)) pel=&((*pel)->next);

        if(!(*pel)) {
            (*pel)=new EfficiencyList;
            (*pel)->next=0;

            el=(*pel);
        }
        else if((*pel)->Q>q) {
            el=new EfficiencyList;

            el->next=(*pel);
            (*pel)=el;
        }
        else el=(*pel);

        el->Changed=1;
        el->Q=(unsigned short)q;
        el->P=SourceDataLines[i].P;
        el->E=q*SourceDataLines[i].P;
        el->Source=(unsigned short)Source;
        el->SourceQ=0;
        el->Distance=1;

        if(SourceDataLines[i].Destination==Destination) if(el->E>minE) minE=el->E;
    }

    while(Change) {
        Change=false;

        for(j=0;j<NumDataLines;j++) if(DataLines[j].active) {
            if(DataLines[j].E<minE) {
                DataLines[j].active=0;

                continue;
            }

            s=DataLines[j].Source;
            d=DataLines[j].Destination;
            p=DataLines[j].P;
            q=DataLines[j].Q;

            pel=&Table[DataLines[j].Destination];
            el=*pel;

            pel2=&Table[DataLines[j].Source];
            el2=*pel2;

            while(el2&&(el2->Q<q)) {
                while(el&&(el->next)&&(el->next->Q<el2->Q)) {
                    if(el->E<minE) {
                        (*pel)=el->next;
                        delete el;

                        el=*pel;
                    }
                    else {
                        pel=&(el->next);
                        el=el->next;
                    }
                }

                if(el&&(el->Q<el2->Q)) {
                    if(el->E<minE) {
                        (*pel)=el->next;
                        delete el;

                        el=*pel;
                    }
                    else {
                        pel=&(el->next);
                        el=el->next;
                    }
                }

                if(el2->E<minE) {
                    (*pel2)=el2->next;
                    delete el2;

                    el2=*pel2;

                    continue;
                }

                if((el2->E*p)<minE) {
                    pel2=&(el2->next);
                    el2=el2->next;

                    continue;
                }

                if(!el) {
                    el=new EfficiencyList;

                    el->P=0.0;
                    el->Q=el2->Q;
                    el->next=0;
                    (*pel)=el;
                }
                else if(el->Q>el2->Q) {
                    el=new EfficiencyList;

                    el->P=0.0;
                    el->Q=el2->Q;
                    el->next=(*pel);
                    (*pel)=el;
                }

                if((el2->Changed==pc)||(el2->Changed==cc)||(el->P==0.0)) {
                    if((el2->P*p)>(el->P)) {
                        el->Changed=cc;
                        el->P=(el2->P*p);
                        el->E=el->P*el->Q;
                        el->Source=(unsigned short)s;
                        el->SourceQ=el2->Q;
                        el->Distance=(el2->Distance+1);

                        Change=true;

                        if(d==Destination) minE=el->E;
                    }
                }
                else el2->Changed=0;

                pel2=&(el2->next);
                el2=el2->next;
            }

            if(el2) {
                while(el&&(el->next)&&(el->next->Q<q)) {
                    if(el->E<minE) {
                        (*pel)=el->next;
                        delete el;

                        el=*pel;
                    }
                    else {
                        pel=&(el->next);
                        el=el->next;
                    }
                }

                if(el&&(el->Q<q)) {
                    if(el->E<minE) {
                        (*pel)=el->next;
                        delete el;

                        el=*pel;
                    }
                    else {
                        pel=&(el->next);
                        el=el->next;
                    }
                }

                if(!el) {
                    el=new EfficiencyList;

                    el->P=0.0;
                    el->Q=(unsigned short)q;
                    el->E=0.0;
                    el->next=0;
                    (*pel)=el;
                }
                else if(el->Q>q) {
                    el=new EfficiencyList;

                    el->P=0.0;
                    el->Q=(unsigned short)q;
                    el->E=0.0;
                    el->next=(*pel);
                    (*pel)=el;
                }

                while(el2) {
                    if((el2->P*q*p)<minE) {
                        pel2=&(el2->next);
                        el2=el2->next;

                        continue;
                    }

                    if((el2->Changed==pc)||(el2->Changed==cc)||(el->P==0.0)) {
                        if(el2->P*p>el->P) {
                            el->Changed=cc;
                            el->P=(el2->P*p);
                            el->E=el->P*el->Q;
                            el->Source=(unsigned short)s;
                            el->SourceQ=el2->Q;
                            el->Distance=(el2->Distance+1);

                            Change=true;

                            if(d==Destination) minE=el->E;
                        }
                    }
                    else el2->Changed=0;

                    pel2=&(el2->next);
                    el2=el2->next;
                }

                if(el->E==0.0) {
                    (*pel)=el->next;
                    delete el;

                    el=*pel;
                }
            }
        }

        pc=cc;
        cc++;
        if(cc>3) cc=1;
    }
}

void Print()
{
    unsigned short Q;
    unsigned long Distance;
    EfficiencyList *el;
    unsigned short cel=(unsigned short)Destination;
    char tmp[30],*p;
    unsigned long Order[1000];
    unsigned long i,j;
    FILE *ouf;

    el=Table[Destination];
    while(el->E!=minE) el=el->next;

    Q=el->Q;
    Distance=el->Distance;

    for(i=0,j=Distance;i<Distance;i++) {
        Order[j--]=cel;
        
        while(el->Q!=Q) el=el->next;

        cel=el->Source;
        Q=el->SourceQ;

        el=Table[el->Source];
    }

    Order[0]=Source;

    ouf=fopen("NETWORK.OUT","wt");

    sprintf(tmp,"%.20Lf",minE);

    p=tmp;
    while((*p)!='.') p++;

    *(p+7)=0;

    fprintf(ouf,"%s %lu\n",tmp,Distance+1);

    fprintf(ouf,"%lu",Order[0]+1);
    for(i=0;i<Distance;i++) fprintf(ouf," %lu",Order[i+1]+1);

    fprintf(ouf,"\n");

    fclose(ouf);
}

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
    ReadData();

    FindMinE();

    Optimize();

    Solve();

    Print();

    return 0;
}



