#pragma warning(disable: 4786)

#include <vector>
#include <list>
#include <cassert>
#include <iostream>
#include <fstream>
#include <limits>
#include <iomanip>

using namespace std;

const int MAX_DIST = 2048;//"infinite" distance between two routers

inline int min(int i,int j){return i<=j?i:j;}

struct Channel{
	int from;//source router
	int to;//target router
	int Q;//speed
	double P;//safety
	int indF,indT;//second index in the double array. Saved for quick access to the next channel
	bool isBad;//Some channels will be labeled bad in the process of iteration
	Channel(int f,int t,int q,double p,int iF,int iT):
		from(f),to(t),P(p),Q(q),indF(iF),indT(iT),isBad(false){}
};

//Save "integral" Q and P for every link
//of the route to avoid their recalculation (iterating over the head subroute)
struct ChannelX{
	Channel* channel;
	int Q;
	double P;
	ChannelX(Channel* cp=0,int q=0,double p=0):channel(cp),Q(q),P(p){}
};

typedef list<ChannelX*> Route;

inline void append(Route& r, Channel* pc){
	int q=pc->Q;
	double p=pc->P;
	if(r.size()!=0){
		Route::reverse_iterator it = r.rbegin();
		p *= (*it)->P;
		q = min(q,(*it)->Q);
	}
	ChannelX* pcX = new ChannelX(pc,q,p);
	r.push_back(pcX);
}

class Routers{
public:
	Routers(int N,int from,int to):channelsFrom(N+1),channelsTo(N+1),
			From(from),To(to),cicles(N+1),currRoute(),bestRoute(){}

	void addChannel(int f, int t, int p, double q){
		Channel* pch = new Channel(f,t,p,q,channelsFrom[f].size(),channelsTo[t].size());
		channelsFrom[f].push_back(pch);
		pch = new Channel(f,t,p,q,channelsFrom[f].size()-1,channelsTo[t].size());
		channelsTo[t].push_back(pch);
	}

	void Init(){
		SetDistTo();
		currE = -1;
		bestE = -1;
	}

	int size(){return channelsFrom.size();}

	Route& begin(int from){
		Route* pr = new Route();
		double p=1;
		int q=INT_MAX;
		while(true){
			if(channelsFrom[from].size()>0){
				p *= channelsFrom[from][0]->P;
				q = min(q,channelsFrom[from][0]->Q);
				ChannelX* chp = new ChannelX(channelsFrom[from][0],q,p);
				pr->push_back(chp);
				from = channelsFrom[from][0]->to;
				if(cicles[from]==1){
					pr->pop_back();
					break;
				}
				else
					cicles[from] = 1;
				if(from==To) break;
			}
			else
				break;
		}

		return *pr;
	}

	Route& begin(){
		Route* pr = new Route();
		if(distTo[From]==MAX_DIST) return *pr;//No routes from From to To
		int from = From;
		cicles[from] = 1;
		while(true){
			if(channelsFrom[from].size()>0){
				vector<Channel*>* pc = &channelsFrom[from];
				int i=0;
				while(i<pc->size()&&(cicles[channelsFrom[from][i]->to]==1||distTo[(*pc)[i]->to]==MAX_DIST||(*pc)[i]->isBad))i++;
				if(i==pc->size()) break;
				append(*pr,channelsFrom[from][i]);
				from = channelsFrom[from][i]->to;
				//if(cicles[from]==1){
					//pr->pop_back();
					//break;
				//}
				//else
					cicles[from] = 1;
				if(from==To) break;
			}
			else
				break;
			Route::reverse_iterator it = pr->rbegin();
			currE= double((*it)->P*(*it)->Q);
			if(currE<=bestE)
				break;
		}
		Route::reverse_iterator it = pr->rbegin();
		currE= double((*it)->P*(*it)->Q);
		if(currE>bestE){
			bestE=currE;
			bestRoute = *pr;
		}
		PrintRoute(*pr);
		return *pr;
	}

	Route& begin(Route& head){
		if(head.size()==0) return head;//No routes more from From to To
		Route* pr = &head;
		Route::reverse_iterator it = pr->rbegin();
		int from = (*it)->channel->to;
		int Q = (*it)->Q;
		double P = (*it)->P;
		while(true){
			if(pr->size()==0) return *pr;
			if(channelsFrom[from].size()>0){
				int i=0;
				vector<Channel*>* pc = &channelsFrom[from];
				while(i<pc->size()&&((double)(min(Q,(*pc)[i]->Q)*P*(*pc)[i]->P)<=bestE||
						cicles[channelsFrom[from][i]->to]==1||distTo[(*pc)[i]->to]==MAX_DIST||(*pc)[i]->isBad))i++;
				if(i==pc->size()) break;
				append(*pr,channelsFrom[from][i]);
				from = channelsFrom[from][i]->to;
				//if(cicles[from]==1){
					//pr->pop_back();
					//break;
				//}
				//else
					cicles[from] = 1;
				if(from==To) break;
			}
			else
				break;
			Route::reverse_iterator it = pr->rbegin();
			currE= double((*it)->P*(*it)->Q);
			if(currE<=bestE)
				break;
		}
		it = pr->rbegin();
		currE= double((*it)->P*(*it)->Q);
		if(currE>bestE&&(*it)->channel->to==To){
			bestE=currE;
			bestRoute = *pr;
		}
		return *pr;
	}

	Route& Next(){
		Route* pr = &currRoute;
		//if(distTo[From]==MAX_DIST) return *pr;//No routes from From to To
		//cout<<"Rootsize: ";
		while(true){
			if(pr->size()==0&&bestE!=-1) return *pr;//No more routes from From to To
			Route::reverse_iterator it = pr->rbegin();
			int from = (*it)->channel->from;
			int i = (*it)->channel->indF+1;
			int Q = (*it)->Q;
			double P = (*it)->P;
			vector<Channel*>* pc = &channelsFrom[from];
			while(i<pc->size()&&((double)(min(Q,(*pc)[i]->Q)*P*(*pc)[i]->P)<=bestE||
					cicles[channelsFrom[from][i]->to]==1||distTo[(*pc)[i]->to]==MAX_DIST||(*pc)[i]->isBad))i++;
			if(i==pc->size()){
				cicles[(*it)->channel->to] = 0;
				pr->pop_back();
				//cout<<pr->size()<<" ";
				continue;
			}
			else{
				//cout<<pr->size()<<" Next router: "<<channelsFrom[from][i]->to<<endl;
				cicles[(*it)->channel->to] = 0;
				pr->pop_back();
				append(*pr,channelsFrom[from][i]);
				cicles[channelsFrom[from][i]->to] = 1;
				begin(*pr);
				break;
			}
		}
		//PrintRoute(*pr);
		return *pr;
	}

	vector<int>* GetPath(const Route& r){
		if(r.size()==0)return 0;
		vector<int>* path = new vector<int>(r.size()+1);
		Route::iterator it = r.begin();
		(*path)[0] = (*it)->channel->from;
		int i=1;
		while(it!=r.end()){
			(*path)[i++] = (*it)->channel->to;
			++it;
		}
		return path;
	}

	void PrintRoute(Route& r){
		vector<int>* path = GetPath(r);
		cout<<endl<<"Path: ";
		for(int i=0; i<path->size(); i++)
			cout<<(*path)[i]<<" ";
		delete path;
		Route::reverse_iterator it = r.rbegin();
		double E = -1;
		if(it!=r.rend())
			E=(double)((*it)->Q*(*it)->P);
		cout<<" E="<<E<<endl;
	}

	Route& GetBestRoute(){
		currRoute = begin();
		bestRoute = currRoute;
		while(currRoute.size()>0)
			Next();
		return bestRoute;
	}

	void SetDistTo(){
		vector<int>::iterator it = distTo.end();
		distTo.resize(0);
		distTo.insert(it,size(),MAX_DIST);
		distTo[To] = 0;
		vector<vector<int> > distTo2(size()-1);
		distTo2[0].push_back(To);
		int added = 1;
		for(int i=0; i<size()-1; i++){
			bool addedNew = false;
			for(int j=0; j<distTo2[i].size(); j++){
				vector<Channel*>* chann = &channelsTo[distTo2[i][j]];
				for(int k=0; k<chann->size(); k++){
					int from = (*chann)[k]->from;
					if(distTo[from]==MAX_DIST&&!(*chann)[k]->isBad){
						distTo2[i+1].push_back(from);
						distTo[from] = i+1;
						addedNew = true;
					}
				}
			}
			if(addedNew==false) break;//remaining routers are unaccessible
		}
	}

	int DistTo(int i){return distTo[i];}

	Route currRoute,bestRoute;
	double currE, bestE;
	int From, To;//source and target routers
protected:
	vector<vector<Channel*> > channelsFrom;
	vector<vector<Channel*> > channelsTo;
	vector<int> cicles;//indicator of routes with loops
	//vector<int> distFrom;//minimal distances from From router
	vector<int> distTo;//minimal distances from To router
};

int main()
{
#if 0
#if 0
	//OK
	Routers routers(7,1,5);
	routers.addChannel(1,2,3,1.0);
	routers.addChannel(2,1,3,1.0);
	routers.addChannel(1,3,3,1.0);
	routers.addChannel(1,4,3,1.0);
	routers.addChannel(1,5,3,1.0);
	routers.addChannel(2,3,3,1.0);
	//routers.addChannel(2,5,3,1.0);
	//routers.addChannel(2,1,3,1.0);
	//routers.addChannel(3,1,2,0.5);
	//routers.addChannel(3,2,3,1.0);
	routers.addChannel(3,4,3,1.0);
	routers.addChannel(4,6,3,1.0);
	routers.addChannel(4,5,2,0.5);
	int i;
#endif
#if 0
	//OK, but rather slow
	int N=300;
	Routers routers(N,1,N);
	int i,j;
	for(i=1;i<=N;i++)
	for(j=1;j<=N;j++)
	if(i!=j)
	routers.addChannel(i,j,2,1.0);
#endif
	routers.Init();
	Route best = routers.GetBestRoute();
	routers.PrintRoute(best);
	//cout<<endl<<"Distances: "<<endl;
	//for(i=1; i<routers.size(); i++)
		//cout<<routers.DistTo(i)<<endl;
#endif
	ifstream is("NETWORK.INP");
	if(!is){
		cout<<"Can not open Catalog.inp!"<<endl;
		exit(1);
	}
	int N, M;
	is>>N>>M;
	Routers routers(N,1,N);
	while(!is.eof()){
		int from,to;
		int Q;
		double P;
		for(int i=0;i<M;i++){
			is>>from>>to>>Q>>P;
			routers.addChannel(from,to,Q,P);
		}
		is>>from>>to;
		routers.From = from;
		routers.To = to;
	}
	is.close();

	ofstream os("NETWORK.OUT");
	if(!os){
		cout<<"Can not open Catalog.out!"<<endl;
		exit(1);
	}

	routers.Init();
	Route best = routers.GetBestRoute();
	//routers.PrintRoute(best);

	if(best.size()==0){
		os<<0<<" "<<0<<endl<<endl;
		return 0;
	}
	os<<setiosflags(ios::fixed)<<routers.bestE<<" "<<best.size()+1<<endl;
	vector<int>* path = routers.GetPath(best);
 	for(int i=0; i<path->size(); i++){
		os<<(*path)[i];
		if(i<path->size()-1)
			os<<" ";
	}
	delete path;
	os.close();
	return 0;
}